<?php
/*
Plugin Name: Woocommerce Wintax Integration
Description: Wintax számlázó és készletkezelő összeköttetés WooCommerce-hez
Author: Novitax Kft.
Author URI: http://www.novitax.hu
Version: 1.1
*/



if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

define('WINTAX_DIR', 'wc_wintax');

// plugin activation: make upload dir fo file exchange

function wc_wintax_activate() {
	$upload_dir =  wp_upload_dir();

	$files = array(
		array(
			'base' 		=> $upload_dir['basedir'] . '/' . WINTAX_DIR,
			'file' 		=> 'index.html',
			'content' 	=> ''
		)
	);

	foreach ( $files as $file ) {
		if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
			if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
				fwrite( $file_handle, $file['content'] );
				fclose( $file_handle );
			}
		}
	}

	
}

register_activation_hook( __FILE__, 'wc_wintax_activate' );


class WC_Wintax {


	public function __construct() {

		// beállítások (tab, űrlap, mentés, scriptek)
		add_filter( 'woocommerce_settings_tabs_array', array( $this, 'add_settings_tab'), 50 );
		add_action( 'woocommerce_settings_tabs_settings_tab_wintax', array( $this, 'settings_tab') );
		add_action( 'woocommerce_update_options_settings_tab_wintax', array( $this, 'update_settings') );
		add_action( 'admin_enqueue_scripts', array( $this, 'load_js') );

		
		// teljesített rendeléskor: számlázáshoz szükséges xml generálás, meta_option _wintax_invoice set to 'new'
		add_action( 'woocommerce_order_status_completed', array( $this, 'on_order_complete' ) );

		// wintaxból jövő hívások kezelése: kiállítandó számlák lekérdezése (-> 'sent'), készlet adatok frissítése, számlaszám visszatöltés (-> 'generated')
		add_action( 'init', array( $this, 'wc_wintax_process' ) );

		// Display field value on the order edit page
		add_action( 'woocommerce_admin_order_data_after_order_details', array( $this, 'show_invoice_num' ), 10, 1 );

  	}

    /**
     * Add a new settings tab to the WooCommerce settings tabs array.
     *
     * @param array $settings_tabs Array of WooCommerce setting tabs & their labels, excluding the Subscription tab.
     * @return array $settings_tabs Array of WooCommerce setting tabs & their labels, including the Subscription tab.
     */
    public static function add_settings_tab( $settings_tabs ) {
        $settings_tabs['settings_tab_wintax'] = __( 'Wintax Integráció', 'wc-wintax' );
        return $settings_tabs;
    }



    /**
     * Uses the WooCommerce admin fields API to output settings via the @see woocommerce_admin_fields() function.
     *
     * @uses woocommerce_admin_fields()
     * @uses self::get_settings()
     */
    public static function settings_tab() {
        woocommerce_admin_fields( self::get_settings() );
    }


    /**
     * Uses the WooCommerce options API to save settings via the @see woocommerce_update_options() function.
     *
     * @uses woocommerce_update_options()
     * @uses self::get_settings()
     */
    public static function update_settings() {
        woocommerce_update_options( self::get_settings() );
    }


    public function load_js($hook) {
	    // Load only on ?page=wc-settings
        if($hook != 'woocommerce_page_wc-settings') {
	        return;
	    }
	    wp_enqueue_script( 'wintax-script', plugins_url('/js/wintax.js', __FILE__)  );
    }


	//Settings
	public static function get_settings( ) {

		$settings = array(
			'section_title' => array(
                'name'     => __( 'Wintax beállítások', 'wc-wintax' ),
                'type'     => 'title',
                'id'       => 'woocommerce_wintax_section_title'
            ),
			'api_key' => array(
				'title'     	=> __( 'Wintax API Key', 'wc-wintax' ),
				'type'  		=> 'text',
				'desc'       	=> __( 'Ezt az API key-t kell magadni a Wintaxban.', 'wc-wintax' ),
				'desc_tip'      => true,
				'default'       => '',
				'id'   			=> 'wc_wintax_key',
				'css'      		=> 'width:300px; margin-right: 20px;',
			),
		
			'section_end' => array(
                 'type' 	=> 'sectionend',
                 'id' 		=> 'woocommerce_wintax_section_end'
            ),
		);

        return $settings;
	}


	//Autogenerate invoice
	public function on_order_complete( $order_id ) {

		//Only generate invoice, if it wasn't already sent 
	
		$invoice_state = get_post_meta($order_id, '_wintax_invoice', true);
		if (in_array($invoice_state, array('new', ''))) {

			// a felesleges xml törlése
			if ($invoice_state == 'new') {

				$UploadDir = wp_upload_dir();
				$UploadURL = $UploadDir['basedir'];
				$location  = realpath($UploadURL . '/' . WINTAX_DIR);
				$xmlfile = $location . '/' . $this->get_xml_filename($order_id);
				@unlink($xmlfile);

			}

			$this->generate_invoice($order_id);
			
			//Store as a custom field
			update_post_meta( $order_id, '_wintax_invoice', 'new');
		}
	
	
	}


	//Generate XML 
	
	public function generate_invoice($orderId) {
		

		$order = new WC_Order($orderId);
		$order_items = $order->get_items();

		//Build Xml

		$o = new SimpleXMLElement('<Order></Order>');
		
		$o->addChild('Key', $order->get_order_number());

		//Customer details
		$vevo = $o->addChild('Customer');
		$vevo->addChild('Id',$this->get_order_property('customer_user',$order));

		$addresses = $vevo->addChild('Addresses');

		$invoice = $addresses->addChild('Invoice');
		$invoice->addChild('Name', ($this->get_order_property('billing_company',$order) ? htmlspecialchars($this->get_order_property('billing_company',$order), ENT_XML1, 'UTF-8') : $this->get_order_property('billing_first_name',$order).' '.$this->get_order_property('billing_last_name',$order)) );
		$invoice->addChild('ZIP',$this->get_order_property('billing_postcode',$order));
		$invoice->addChild('City',$this->get_order_property('billing_city',$order));
		$invoice->addChild('Street',$this->get_order_property('billing_address_1',$order));
		$invoice->addChild('StreetName',$this->get_order_property('billing_address_1',$order));

		$invoice->addChild('County','');
		$c = WC()->countries->countries[ $this->get_order_property('billing_country',$order) ];
		$invoice->addChild('Country',$c);
		$invoice->addChild('CountryCode',$this->get_order_property('billing_country',$order));
		//$invoice->addChild('TaxNumber',$this->get_order_property('',$order));
		

		$shipping = $addresses->addChild('Shipping');
		$shipping->addChild('Name', ($this->get_order_property('shipping_company',$order) ? htmlspecialchars($this->get_order_property('shipping_company',$order), ENT_XML1, 'UTF-8') : $this->get_order_property('shipping_first_name',$order).' '.$this->get_order_property('shipping_last_name',$order)) );
		$shipping->addChild('ZIP',$this->get_order_property('shipping_postcode',$order));
		$shipping->addChild('City',$this->get_order_property('shipping_city',$order));
		$shipping->addChild('Street',$this->get_order_property('shipping_address_1',$order));
		$shipping->addChild('StreetName',$this->get_order_property('shipping_address_1',$order));

		$shipping->addChild('County','');
		$c = WC()->countries->countries[ $this->get_order_property('shipping_country',$order) ];
		$shipping->addChild('Country',$c);
		$shipping->addChild('CountryCode',$this->get_order_property('shipping_country',$order));
		
		// $order_currency = method_exists( $order, 'get_currency' ) ? $order->get_currency() : $order->get_order_currency();
		$o->addChild('Currency', $this->get_order_property('order_currency',$order));
		
		$pt = $this->get_payment_type($this->get_order_property('payment_method',$order), $this->get_order_property('payment_method_title',$order));

		$o->addChild('Payment', $pt);


		$items = $o->addChild('Items');				
		$_tax = new WC_Tax();

		// Order Items
		foreach( $order_items as $item ) {

			// wc 3.0+
			if(method_exists( $order, 'get_id' )) {

				$_product = get_product( $item['variation_id'] ? $item['variation_id'] : $item['product_id'] );

				$i = $items->addChild('Item');
				$i->addChild('Id', $item['product_id']);
				$i->addChild('Sku', $item->get_product()->get_sku());
				$i->addChild('Name',htmlspecialchars($item->get_name()));
				
				$i->addChild('Unit','db');				
				$i->addChild('Quantity',$item->get_quantity());
				
				$i->addChild('PriceGross',round(($item->get_total() + $item->get_total_tax()) / $item->get_quantity(), 2));
				
				$rates = array_shift($_tax->get_rates( $_product->get_tax_class() ));
				$item_rate = round(array_shift($rates));

				$i->addChild('Vat', $item_rate . '%');

			}
			else {

				
				$_product = new WC_Product( $item['variation_id'] ? $item['variation_id'] : $item['product_id'] );

				$i = $items->addChild('Item');
				$i->addChild('Id', $item['product_id']);
				$i->addChild('Sku', $_product->get_sku());
				$i->addChild('Name',htmlspecialchars($item['name']));
				
				$i->addChild('Unit','db');				
				$i->addChild('Quantity',$item['qty']);
				
				$i->addChild('PriceGross',round(($item['line_total'] + $item['line_tax']) / $item['qty'], 2));
				$i->addChild('Vat', round(($item['line_tax']/$item['line_total'])*100) . '%');

			}

		}

		// Shipping
		if($order->get_shipping_methods()) {
			$i = $items->addChild('Item');
			$i->addChild('Id', 'shipping-cost');
			$i->addChild('Sku', 'shipping-cost');
			$i->addChild('Name', 'Szállítási költség - ' . htmlspecialchars($order->get_shipping_method()));
			
			$i->addChild('Unit','db');				
			$i->addChild('Quantity',1);
			
			$order_shipping = method_exists( $order, 'get_shipping_total' ) ? $order->get_shipping_total() : $order->order_shipping;
			$order_shipping_tax = method_exists( $order, 'get_shipping_tax' ) ? $order->get_shipping_tax() : $order->order_shipping_tax;

			$i->addChild('PriceNet',round($order_shipping,2));

			if($order_shipping == 0) {
				$i->addChild('Vat','0%');
			} else {
				$i->addChild('Vat',round(($order_shipping_tax/$order_shipping) * 100) . '%');
			}

		}



		$xml = $o->asXML();
		$xml = substr($xml, strpos($xml, '?>') + 2);


		$random = substr(md5(rand()),8);
		update_post_meta( $orderId, '_wintax_xml_filerand', $random);

		// save XML
		$UploadDir = wp_upload_dir();
		$UploadURL = $UploadDir['basedir'];
		$location  = realpath($UploadURL . '/' . WINTAX_DIR);
		
		$xmlfile = $location . '/' . $this->get_xml_filename($orderId);
		file_put_contents($xmlfile, $xml);


	}





	public function wc_wintax_process() {
		
		if (isset($_POST['wintax_method'])) {

			$r = $this->check_key($_POST['wintax_key']);
			if (!$r) {
				header('HTTP/1.1 401 Unauthorized');
				exit;
			}

			switch ($_POST['wintax_method']) {
				case 'update_stock':
					$this->update_stock();
					exit;
				case 'get_invoice_data':
					$this->get_invoice_data();
					exit;
				case 'send_invoice':
					$this->send_invoice();
					exit;											
				case 'test_connection':
					$this->test_connection();
					exit;					
				default:
					header('HTTP/1.0 400 Bad Request');
					exit;
			}
			
		}
		else {
			return;
		}


	}


	// kapcsolat ellenőrzés
	public function test_connection() {
		echo 'OK';
	}



	// készlet adatok frissítése wintax felől
	public function update_stock() {

		
		libxml_use_internal_errors(true);
		$xml = simplexml_load_string(stripslashes($_POST['wintax_xml']));

		if ($xml === false) {
			header('HTTP/1.0 422 Unprocessable Entity');
			echo 'malformed xml';
			exit;
		}

		$found = 0;

		foreach ($xml->StockItem as $product) {


			$sku = (string)$product->EAN;
			$price = (int)$product->Price;
			$q = (int)$product->Amount;
	
 			if (!empty($sku)) {
 				$product_id = wc_get_product_id_by_sku($sku);
 				if ($product_id) {
					$found++;
					wc_update_product_stock($product_id, $q);
					update_post_meta($product_id, '_regular_price', $price);
					update_post_meta($product_id, '_price', $price);
				}
			}
		}

		echo 'OK: ' . $found;
	}

	// számlázandó rendelések xml-jeinek összefűzése és elküldése
	public function get_invoice_data() {
		
		/*
		$args = array(
		    'meta_key'    => '_wintax_invoice',
    		'meta_value'  => 'new',
		);

		$to_export = wc_get_orders($args);
		*/


		$to_export = get_posts( array(
		    'meta_key'    => '_wintax_invoice',
    		'meta_value'  => 'new',
		    'post_type'   => wc_get_order_types(),
		    'post_status' => array_keys( wc_get_order_statuses() ),
		));


		$xml = '<?xml version="1.0" encoding="UTF-8"?><Orders>';

		$UploadDir = wp_upload_dir();
		$UploadURL = $UploadDir['basedir'];
		$location  = realpath($UploadURL . '/' . WINTAX_DIR);
		
		
		foreach ($to_export as $t) {

			//$order_id = $t->get_order_number();
			$order_id = $t->ID;

			$xmlfile = $location . '/' . $this->get_xml_filename($order_id);
			$xml .= file_get_contents($xmlfile);
			
			update_post_meta($order_id, '_wintax_invoice', 'sent');
			$order = new WC_Order($order_id);
			$order->add_order_note( __( 'Számlázási adatok átadva Wintaxnak', 'wc-wintax' ) );

		}

		$xml .= '</Orders>';
		
		header("Content-type: text/xml; charset=utf-8");
		echo $xml;
		

	}

	// számlázott rendelések számlaszámának (pdf számláinak) fogadása
	public function send_invoice() {

		$order_id = (int)$_POST['order_id'];
		$invoice_num = $_POST['invoice_num'];

		//$file = $_POST['invoice_pdf'];

		try {

			$order = new WC_Order($order_id);
			update_post_meta($order_id, '_wintax_invoice', 'generated');
			update_post_meta($order_id, '_wintax_invoice_id', $invoice_num);

			$order->add_order_note( __( 'WINTAX: számlázás sikeres.', 'wc-wintax' ) );
			
			$UploadDir = wp_upload_dir();
			$UploadURL = $UploadDir['basedir'];
			$location  = realpath($UploadURL . '/' . WINTAX_DIR);
			$xmlfile = $location . '/' . $this->get_xml_filename($order_id);
			@unlink($xmlfile);

			echo 'OK';

		}
		catch (Exception $e) {
			header('HTTP/1.0 422 Unprocessable Entity');
			echo 'order_id not exists: ' . $order_id . "\n";
			exit;	
		}


	}


	public function show_invoice_num($order){
		$szla_szam = get_post_meta( $order->id, '_wintax_invoice_id', true );
	    if (!empty($szla_szam)) echo '<p class="form-field form-field-wide wc-customer-user"><strong>'.__('Wintax által generált számlaszám').':</strong> <br/>' .  $szla_szam . '</p>';
	}


	// fizetés módja
	public function get_payment_type($type, $title) {
		
		switch ($type) {
			case 'bacs':	// átutalás
				return 'AT';
				break;
			case 'cod':		// utánvétel
				return 'UV';
				break;			
			default:
				return 'other - ' . $title;
				break;
		}

	}



	// letárolt xml fájlnév $order_id alapján
	public function get_xml_filename($order_id) {
		
		$random = get_post_meta($order_id, '_wintax_xml_filerand', true);
		return $order_id . '-' . $random . '.xml';

	}



	// kulcs összevetése a tárolttal
	public function check_key($key) {
		if (!empty($key) && get_option('wc_wintax_key') == $key) {
			return true;
		}
		else {
			return false;
		}
	}

	//Get order ID(backward compatibility), for WC3.0+
	public function get_order_id($order) {
		$id = method_exists( $order, 'get_id' ) ? $order->get_id() : $order->id;
		return $id;
	}

	//Get order details(backward compatiblity), for WC3.0+
	public function get_order_property($property,$order) {

		//3.0+
		$value = '';
		if(method_exists( $order, 'get_'.$property )) {
			$property = 'get_'.$property;
			$value = $order->$property();
		} else {
			$value = $order->$property;
		}

		return $value;

	}


}

$GLOBALS['wc_wintax'] = new WC_Wintax();

